//**************************************************************************************************
//*  FRIDA: fast reliable interactive data analysis
//*  (C) Joachim Wuttke 1990-, v2(C++) 2001-
//*  http://apps.jcns.fz-juelich.de/frida
//**************************************************************************************************

//! \file  dualplot.hpp
//! \brief Declares class CPlot.

#include <memory>
#include "axis.hpp"

//! One plot frame.

class CPlot
{
public:
    int iPlot; //!< The index of this frame in NPloWin::Plots.
    CAxis X; //!< Limits, log flag &c for x axis.
    CAxis Y; //!< Limits, log flag &c for y axis.
    // For data plotting:
    int maxpoints; //!< Maximum # points to be plotted without reduction.
    bool with_errors; //!< Plot error bars?
    // For curve plotting:
    int equipoints; //!< Start curve plot with this # grid points.
    bool refine; //!< Refine curve plot when appropriate?

    CPlot(int _iPlot, bool _logx, bool _logy);

    void gp_write(const std::string& in);
    void clear_frame();
    void plot_frame(const std::string& xlabel, const std::string& ylabel);
    void add_spec(
        bool as_line, bool new_slice, int style_no, const std::vector<double>& xp,
        const std::vector<double>& yp, const std::vector<double>& dyp,
        const std::vector<std::string>& zentries, const std::string& xco, const std::string& yco,
        const std::string& info);
    void show_specs();
    void doc_TxLine(const std::string& line);
    void doc_PtTxLine(const std::string& line, int num);
    void doc_CvTxLine(const std::string& line, int num);
    void write_postscript(
        const std::string& mode, const std::string& fname, const std::string& ps_outdir,
        const std::string& ps_head, const std::string& ps_cont, const std::string& ps_dict);
    void set_aux(const std::string& cmd);
    std::string info() const;

private:
    class CGnuPlotter* gnuPlotter;
    class CPS_Plotter* ps_Plotter;
};
