//**************************************************************************//
//* FRIDA: fast reliable interactive data analysis                         *//
//* (C) Joachim Wuttke 1990-, C++ rewrite 2001-                            *//
//* http://apps.jcns.fz-juelich.de/frida                                   *//
//**************************************************************************//

//! \file  frida2.cpp
//! \brief Main program: initialize, then run interactive endless loop.


//! \mainpage FRIDA: fast reliable interactive data analysis

//! \par Homepage:
//!           http://apps.jcns.fz-juelich.de/frida

//! Useful entry points in this source documentation:
//! - initializations, main loop:
//!     <a href="frida2_8cpp_source.html">frida2.cpp</a>
//! - commands:
//!     <a href="commands_8cpp_source.html">commands.cpp</a>
//! - online memory:
//!     <a href="namespaceNOlm.html">namespace NOlm</a>
//! - data model root:
//!     <a href="classCOlo.html">class COlo</a>
//!
//! Used libraries:
//! - Minor utilities:
//!     <a href="../frida-libs/trivia/index.html">libtrivia</a>
//! - Dialogue interface:
//!     <a href="../frida-libs/readplus/index.html">libreadplus</a>


#include <string>
#include <vector>
#include <tclap/CmdLine.h>
#include "config_src.hpp"
#include "toplevel.hpp"

//! Frida main program.

int main(int argc, char* argv[])
{
    try {
	TCLAP::CmdLine cmd("Flexible rapid interactive data analysis", ' ', VERSION, true);
	TCLAP::SwitchArg batchSwitch("b","batch","Batch mode: exit on error", cmd, false);
        TCLAP::UnlabeledMultiArg<std::string> multi("script", "Script files", false, "");
        cmd.add( multi );
        cmd.parse(argc, argv);

	bool batchMode = batchSwitch.getValue();
        std::vector<std::string> scripts = multi.getValue();

        // Initialize runtime environment.
        CFrida frida;

#ifdef CONFDIR
        // Execute initialization script.
        frida.execute_file( CONFDIR "/frida.ini", batchMode );
#endif

        // Execute scripts from command line (used in particular by shebang files).
        for( const std::string& script: scripts )
            frida.execute_file( script, batchMode );

        // Interactive main loop.
        frida.interactive();

    } catch (TCLAP::ArgException &e) {
        std::cerr << "error: " << e.error() << " for arg " << e.argId() << "\n";
        exit(1);
    } catch (...) {
        std::cerr << "Unexpected uncatched exception in main program\n";
        exit(1);
    }
    exit(0);
}
