//**************************************************************************************************
//* libtrivia: various little routines for C++
//* vector_ops: vector operations
//* (C) Joachim Wuttke 2001, 2012-
//* http://apps.jcns.fz-juelich.de
//**************************************************************************************************

#ifndef VECTOR_OPS_H
#define VECTOR_OPS_H

#include <vector>
#include <functional>
#include <algorithm>

namespace triv
{
    void insert_in_sorted(std::vector<double>* V, double val);
    void make_unique(std::vector<double>* V, double tolabs = 1e-100, double tolrel = 1e-10);
    bool is_ascending(const std::vector<double>& V);
    bool is_equidist(double* step, const std::vector<double>& V);
    std::vector<size_t> sorted_indices(std::vector<double> const& V);
    std::vector<int> iota_list(int number_of_entries);
    std::vector<double> histogram_limits(const std::vector<double>& channel_centers);
    void increment_indices(std::vector<int>& v, int incr, int siz);
    std::string indices_to_s(const std::vector<int>& v);

    template <class T> void getMinMax(const std::vector<T>& v, T& minval, T&maxval);
    template <class T> bool contains(const std::vector<T>& v, const T e);
    template <class T, class Pred> std::vector<T> merge_sorted(
        const std::vector<T>& a, const std::vector<T>& b, Pred a_before_b);

//**************************************************************************************************
//  Template implementation
//**************************************************************************************************

    template <class T>
    void getMinMax(const std::vector<T>& v, T& minval, T&maxval)
    {
        if (v.size()==0)
            throw "BUG: attempt to determine min and max of empty vector";
        minval = v.front();
        maxval = v.front();
        for (auto t = v.begin()+1; t<v.end(); ++t) {
            minval = std::min(minval, *t);
            maxval = std::max(maxval, *t);
        }
    }

    template <class T>
    bool contains(const std::vector<T>& v, const T e)
    {
        return std::find(v.begin(), v.end(), e) != v.end();
    }

    template <class T, class Pred>
    std::vector<T> merge_sorted(const std::vector<T>& a, const std::vector<T>& b, Pred a_before_b)
    {
        std::vector<T> ret;
        auto ia=a.begin();
        auto ib=b.begin();
        for (;ia!=a.end() && ib!=b.end();)
            ret.push_back( a_before_b(*ia,*ib) ? *(ia++) : *(ib++) );
        for (;ia!=a.end();)
            ret.push_back( *(ia++) );
        for (;ib!=b.end();)
            ret.push_back( *(ib++) );
        return ret;
    }
}

#endif // VECTOR_OPS_H
